#include <mask.h>
#include <form.h>
#include <printapp.h>
#include <utility.h>

#include "lf.h"
#include "soggetti.h"
#include "sezioni.h"
#include "contsan.h"

#include "at4.h"
#include "at4100a.h"

#define ALIAS_LCP	100
#define ALIAS_TCS	200

enum ts { undefined = 0, elenco = 1, etichette = 2 };

// definizione form per etichette
class TSitsan_form : public TForm
{
public:
	
	virtual TCursor* cursor() const;
	virtual TRelation* relation() const;
	TPrint_section& get_body() { return section('B'); } ;
  TSitsan_form(): TForm() {};
  TSitsan_form(const char* form, const char * code = "", int editlevel = 0, const char* desc = "")
    						: TForm(form,code,editlevel,desc) {};
  virtual ~TSitsan_form() {};
};

class TSitsan_application : public TPrintapp
{
  TRelation*				_rel;
  TMask*      			_msk;
  TSitsan_form*			_form_eti;
  
  int 							_cur1, _cur2, _cur3, _cur4;
  TParagraph_string _cognome_nome;
  TDate 						_data_stampa;
	ts 	      				_tipo_stampa;
	TString						_codsez, _codsot;

	bool 							_usotipo;	
	TString						_tipo;
	long							_codmed;

  static bool filter_func_sitsan(const TRelation *);
  
protected:
  virtual bool user_create();
  virtual bool user_destroy();
  virtual bool set_print(int m);
  virtual void set_page(int file, int cnt);
  virtual bool preprocess_page(int file, int counter);

public:
  void crea_intestazione();
  void filtra_sezioni();
  void header_sezione(TString codsez, TString codsot);
  TMask& app_mask() { return *_msk; }
  TSitsan_application() : _data_stampa(TODAY), _cognome_nome("",25) {}
};

HIDDEN inline TSitsan_application& app() { return (TSitsan_application&) main_app(); }

TCursor* TSitsan_form::cursor() const { return app().current_cursor(); }

TRelation* TSitsan_form::relation() const { return cursor()->relation(); }

void TSitsan_application::filtra_sezioni()
{
 	TString sezini = _msk->get(F_SEZINI);
 	TString sotini = _msk->get(F_SOTINI);
 	TString sezfin = _msk->get(F_SEZFIN);
 	TString sotfin = _msk->get(F_SOTFIN);
  select_cursor(_cur4); 
  TLocalisamfile& fl = current_cursor()->file(LF_SOGGETTI);
  TRectype da(fl.curr());
  TRectype a(fl.curr());   
  da.zero();
  a.zero();
  if ((sezini.not_empty()) && (sezini.ok()))
		da.put(SOG_CODSEZ, sezini);
  if ((sotini.not_empty()) && (sotini.ok()))    	
		da.put(SOG_CODSOT, sotini);
  if ((sezfin.not_empty()) && (sezfin.ok()))
		a.put(SOG_CODSEZ, sezfin);
  if ((sotfin.not_empty()) && (sotfin.ok()))    	
		a.put(SOG_CODSOT, sotfin);
	current_cursor()->setregion(da, a);
}	

void TSitsan_application::set_page(int file, int cnt)
{
	// costruzione etichette
	switch (_tipo_stampa)
	{
		case etichette:
		{
			TPrint_section& corpo = _form_eti->get_body();
			corpo.reset();
			corpo.update();
  		for (int i = 0; i < corpo.height(); i++)
  		{
  			TPrintrow& riga = corpo.row(i);
  			set_row(i+1,riga);
			}  			
			force_setpage(TRUE);	// serve perch alla prossima etichetta rifaccia la setpage
														// altrimenti stampa sempre la stessa etichetta
		}
		break;
		case elenco:
		{
			set_row(1,"@0g@pn", FLD(LF_SOGGETTI,SOG_CODICE,"######"));
			set_row(1,"@7g@S", FLD(LF_SOGGETTI,SOG_CATDON));
			set_row(1,"@10g#a", &_cognome_nome);
			set_row(1,"@36g@ld", FLD(LF_SOGGETTI,SOG_DATANASC));
			set_row(1,"@47g@ld", FLD(LF_SOGGETTI,SOG_DATAULTID));	// potremmo mettere anche datastato
			set_row(1,"@58g@S", FLD(LF_SOGGETTI,SOG_IDON1));
			set_row(1,"@61g@S", FLD(LF_SOGGETTI,SOG_IDON2));
			set_row(1,"@64g@S", FLD(LF_SOGGETTI,SOG_IDON3));
			set_row(1,"@67g@S", FLD(LF_SOGGETTI,SOG_IDON4));

			set_row(2,"@50g@S", FLD(LF_SOGGETTI,SOG_STATO));
		}
		break;
	}					
}

bool TSitsan_application::filter_func_sitsan(const TRelation * rel)
{   

	bool filtrato = TRUE;
	
	TLocalisamfile*      sog = &(rel->lfile(LF_SOGGETTI)); 

	TAssoc_array categorie;

	TMask& msk = app().app_mask();

  TString cat = msk.get(F_CAT1);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
  cat = msk.get(F_CAT2);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
  cat = msk.get(F_CAT3);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
  cat = msk.get(F_CAT4);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
  cat = msk.get(F_CAT5);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
  cat = msk.get(F_CAT6);
  if (cat.not_empty() && cat.ok())
  	categorie.add((const char*) cat);
	
	if (categorie.items() != 0)
	{
		TString catsog = sog->curr().get(SOG_CATDON);
		if (categorie.is_key((const char*) catsog))
			filtrato = TRUE;
		else
			filtrato = FALSE;	
	}	
	if (filtrato)
	{
		const long codice = sog->curr().get_long(SOG_CODICE);
		const int prog = 999;		
		TLocalisamfile con(LF_CONTSAN);
		con.zero();
		con.put(CON_CODICE,codice);
		con.put(CON_PROGCON,prog);
		int err = con.read(_isgteq);
		if (err == _isemptyfile)
			filtrato = FALSE;
		else
		{
			if (err==NOERR)
			{
				const long codcon = con.get_long(CON_CODICE);
				if (codice != codcon)
				{
					err = con.prev();
					if (err != NOERR)
						filtrato = FALSE;
				}					
		  } 
		  if (filtrato)
		  {
		  	const long codsog = con.get_long(CON_CODICE);
		  	if (codice != codsog)
		  		filtrato = FALSE;
				else if (app()._usotipo)
				{						
					TString tipo = con.get(CON_TIPOCON);
					if (tipo == app()._tipo)
						filtrato = TRUE;
					else
						filtrato = FALSE;																	
				}							
			}				
		}			
	}
  return filtrato;
}

bool TSitsan_application::preprocess_page(int file, int counter)
{
	// contatore soggetti stampati 
	// per ora non c'
	if (_tipo_stampa == elenco)
	{
		TString nome = current_cursor()->curr().get(SOG_COGNOME);
		nome << " ";
		nome << current_cursor()->curr().get(SOG_NOME);
		_cognome_nome = nome;

		// salto pagina se cambio sezione
		TString codsez = current_cursor()->curr().get(SOG_CODSEZ);		
		TString codsot = current_cursor()->curr().get(SOG_CODSOT);		
		if ((_codsez!=codsez)||(_codsot!=codsot))
		{
			//if (!_codsez.blank())
			if (_codsez != "**")
				printer().formfeed();
			_codsez = codsez;
			_codsot = codsot;
			header_sezione(codsez, codsot);
		}
	}		
  return TRUE;
}

void TSitsan_application::header_sezione(TString codsez, TString codsot)
{
	TString densez = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSEZ);
	TString densot = current_cursor()->curr(LF_SEZIONI).get(SEZ_DENSOT);
	TString intestazione(132);
	intestazione = "Sezione: ";
	intestazione << codsez;
	intestazione << "/";
	intestazione << codsot;
	intestazione << " ";
	intestazione << densez;
	if ((densot.ok())&& (densot.not_empty()))
	{
		intestazione << "/";
		intestazione << densot;
	}		
	intestazione.center_just();
	set_header(1,"@0g%s", (const char*) intestazione);
	return;
}

bool TSitsan_application::set_print(int m)
{ 
	_tipo_stampa = undefined;
  KEY tasto;
  tasto = _msk->run();
  switch (tasto)
  {
  case F_ELENCO:
  	_tipo_stampa = elenco;
  	//_codsez.spaces(2);
  	//_codsot.spaces(2);
  	_codsez = "**";
  	_codsot = "**";
  	break;
  case F_ETICHETTE:
  	_tipo_stampa = etichette;	
  	break;
  }	
	if (_tipo_stampa != undefined)
  {
    reset_files(); 
    add_file(LF_SOGGETTI);
                              
		_usotipo = _msk->get_bool(F_USOTIPO);                              		
    _tipo = _msk->get(F_TIPO);
    _codmed = _msk->get_long(F_CODMED);
    
    // filtro per sezioni selezionati
	  filtra_sezioni();
                                     
		// filtro per tipo idoneit
		/*
    TString tipoid = _msk->get(F_TIPO);
    if (tipoid.not_empty() && tipoid.ok())
    	current_cursor()->setfilter(format("STATO == \"%s\"",(const char*)tipoid));
 		else      
      current_cursor()->setfilter("TCS->S6 == \"I\"", TRUE);
		*/		
		// filtro per categorie
		current_cursor()->set_filterfunction(filter_func_sitsan);
		reset_print();
		crea_intestazione();
    return TRUE;
  }
  else
    return FALSE;
}

void TSitsan_application::crea_intestazione()
{
  reset_header();
  
  if (_tipo_stampa == elenco)
  {
  	TString sep(132);
  	sep = "SITUAZIONE SANITARIA ATTUALE";
  	TString tipo = _msk->get(F_TIPO);
  	if ((tipo.ok()) && (tipo.not_empty()))
  	{
  		sep << ": tipo ";
  		sep << tipo;
  		tipo = _msk->get(F_D_TIPO);
  		sep << " ";
  		sep << tipo;
		}
		sep.center_just();
  	set_header(2, "@0g%s", (const char*) sep);
  	TString data_stampa = _data_stampa.string();
  	set_header(2,"@0g%10s", (const char*) data_stampa);
  	sep = "";
  	sep << "Pag. @#";
  	set_header(2, "@120g%s", (const char*) sep);  
  	
  	sep = "";
  	sep.fill('-');
  	set_header(3, (const char *) sep);

  	set_header(4,"@0gCod.@7gC.@10gCognome e nome@36gNato il@47gData ID@58gTipi idon.");
  	set_header(5,"@47gTipo/Esito");  
  	set_header(6,"@0g------@7g--@10g-------------------------@36g----------@47g----------@58g-- -- -- --");
	}  	
}

bool TSitsan_application::user_create()
{
  _rel = new TRelation(LF_SOGGETTI);
  _rel->add("TCS","CODTAB==STATO",1,0,ALIAS_TCS);
  _rel->add("LCP","CODTAB==LOCALITA",1,0,ALIAS_LCP);
  _rel->add(LF_COMUNI,"COM==COM");
  // per stampare la denominazione della sezione nell'intestazione
  _rel->add(LF_SEZIONI,"CODSEZ==CODSEZ|CODSOT==CODSOT");
	_rel->add(LF_CONTSAN,"CODICE==CODICE");
	
  _cur1 = add_cursor(new TCursor(_rel, "", 1));	//cursore ordinamento per codice
  _cur2 = add_cursor(new TCursor(_rel, "", 2));	//cursore ordinamento per cognome e nome
  _cur3 = add_cursor(new TCursor(_rel, "", 3));	//cursore ordinamento per sezione+sottogruppo+codice
  _cur4 = add_cursor(new TCursor(_rel, "", 4));	//cursore ordinamento per sezione+sottogruppo+cognome e nome

  _msk = new TMask("at4100a");
  
  _form_eti = new TSitsan_form("AT_ETSOG");
  
  return TRUE;
}

bool TSitsan_application::user_destroy()
{
  delete _msk;
  delete _rel;
  delete _form_eti;
  return TRUE;
}

int at4100(int argc, char* argv[])
{

  TSitsan_application a;

  a.run(argc, argv, "Situazione sanitaria attuale");

  return 0;
}
